@@ -1,3 +1,6 @@
+#include "config.h"
+#include "compdefs.h"
+
 #include <stdlib.h>
 #include <wchar.h>
 #include "exitcode.h"
@@ -51,3 +54,22 @@ FillDynamicString(struct dstr *tgt, wchar_t *src) {
   tgt->l = length;
   return(SUCCESS);
 } 
+
+/* Append a wide string to a dynamic wide string */
+int
+AppendToDynamicString(struct dstr *tgt, wchar_t *src) {
+  int SrcLength;
+  int NewLength;
+
+  SrcLength = wcslen(src);
+  NewLength = SrcLength + tgt->l;
+
+  if (NewLength > tgt->c) {
+    tgt->s = (wchar_t *) realloc(tgt->s,(NewLength + 1) * sizeof(wchar_t));
+    if(tgt->s == NULL) return(ERROR);
+    tgt->c = NewLength;
+  }
+  wcscpy(tgt->s+tgt->l,src);
+  tgt->l = NewLength;
+  return(SUCCESS);
+} 
