@@ -1,11 +1,10 @@
-/* Time-stamp: <2005-10-18 03:02:19 poser> */
+/* Time-stamp: <2006-05-21 13:26:57 poser> */
 
 /*
- * Copyright (C) 1993-2003 William J. Poser.
+ * Copyright (C) 1993-2006 William J. Poser.
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of version 2 of the GNU General Public License as published by
+ * the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,24 +18,35 @@
 
 static char rcsid[] = "$Id: misc.c,v 1.10 2003/11/20 02:51:49 poser Exp poser $";
 
+#include "config.h"
+#include "compdefs.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
-#ifdef HAVE_LIBINTL_H
+#ifdef LOCALE_GETTEXT
 #include <libintl.h>
+#define _(x) gettext(x)
 #else
-#define gettext(x) (x)
+#define _(x) (x)
 #endif
 #include <wchar.h>
 #include <errno.h>
+#include "regex.h"
+#include "key.h"
 #include "unicode.h"
 #include "exitcode.h"
 #include "retcodes.h"
 #include "ex_codes.h"
-#include "defs.h"
+
+#ifdef HAVE_LONGDOUBLE
+#define DOUBLE (long double)
+#else
+#define DOUBLE double
+#endif
 
 void
 DumpExclusions(unsigned char *exct,long exctsize,FILE *fp)
@@ -44,15 +54,15 @@ DumpExclusions(unsigned char *exct,long exctsize,FILE *fp)
   unsigned long i;
   unsigned char flags;
 	
-  fprintf(fp,gettext("Characters excluded:\n"));
+  fprintf(fp,_("Characters excluded:\n"));
   if(exctsize > 1) printf("\n");
   for(i = 0L;i < exctsize; i++){
     if(exct[i]){
       flags = exct[i];
       fprintf(fp,"\t0x%04lX",i);
-      if(flags & EX_INITIAL) fprintf(fp,gettext("\tinitial"));
-      if(flags & EX_MEDIAL) fprintf(fp,gettext("\tmedial"));
-      if(flags & EX_FINAL) fprintf(fp,gettext("\tfinal"));
+      if(flags & EX_INITIAL) fprintf(fp,_("\tinitial"));
+      if(flags & EX_MEDIAL) fprintf(fp,_("\tmedial"));
+      if(flags & EX_FINAL) fprintf(fp,_("\tfinal"));
       fprintf(fp,"\n");
     }
   }
@@ -65,7 +75,7 @@ DumpExclusions(unsigned char *exct,long exctsize,FILE *fp)
  */
 
 int
-Exclude(KEYTYPE *str, int len, wchar_t *exct)
+Exclude(wchar_t *str, int len, wchar_t *exct)
 {
   int i;
   int cnt;
@@ -81,26 +91,26 @@ Exclude(KEYTYPE *str, int len, wchar_t *exct)
   if(!(exct[str[len-1]] & EX_FINAL)){
     str[cnt++] = str[len-1];
   }
-  str[cnt] = ((KEYTYPE) 0); /* Null terminate */
+  str[cnt] = ((wchar_t) 0); /* Null terminate */
   return(cnt);
 }
 
 void
-KeyCopy(KEYTYPE *t,KEYTYPE *s,int len)
+KeyCopy(wchar_t *t,wchar_t *s,int len)
 {
   register int i;
   for(i = 0;i < len;i++) *t++ = *s++;
 }
 
 void
-RevKeyCopy(KEYTYPE *t,KEYTYPE *s,int len)
+RevKeyCopy(wchar_t *t,wchar_t *s,int len)
 {
   register int i;
 	
   s += (len-2);
 	
   for(i = 0;i < len-1;i++) *t++ = *s--;
-  *t= (KEYTYPE) 0;
+  *t= (wchar_t) 0;
 }
 
 void
@@ -124,7 +134,7 @@ CopyCommandLine(FILE *fp,int ac, char **av)
 #define DTITYPE unsigned long
 #define SECONDSINDAY (60*60*24)
 int
-GetISO8601Key(wchar_t *field, double *key)
+GetISO8601Key(wchar_t *field, DOUBLE *key)
 {
   unsigned int year = 0;
   unsigned int month =0;
@@ -158,18 +168,18 @@ GetISO8601Key(wchar_t *field, double *key)
 
   Days = (DTITYPE) ( (366 * (DTITYPE) year) + (31 * month) + day);
   DaySeconds = (DTITYPE) (((((DTITYPE) hour * 60) + (DTITYPE) minute) * 60) + (DTITYPE) second);
-  *key = (double) ( ( (double)Days * (double)SECONDSINDAY) + (double)DaySeconds);
+  *key = (DOUBLE) ( ( (DOUBLE)Days * (DOUBLE)SECONDSINDAY) + (DOUBLE)DaySeconds);
   return(SUCCESS);
 }
 
 /* Given a date string, write the equivalent double into key. */
 
 int
-GetDateKey(wchar_t *field, double *key, struct ymdinfo *ymd)
+GetDateKey(wchar_t *field, DOUBLE *key, struct ymdinfo *ymd)
 {
   long int num[3];
-  double month;
-  double day;
+  DOUBLE month;
+  DOUBLE day;
   wchar_t *sepptr1;
   wchar_t *sepptr2;
 
@@ -188,49 +198,92 @@ GetDateKey(wchar_t *field, double *key, struct ymdinfo *ymd)
   if((errno == EINVAL)||(errno == ERANGE)) return (ERROR);
   num[2] = wcstol(sepptr2+1,NULL,10);
   if((errno == EINVAL)||(errno == ERANGE)) return (ERROR);
-  month = (double) num[ymd->m];
-  day = (double) num[ymd->d];
+  month = (DOUBLE) num[ymd->m];
+  day = (DOUBLE) num[ymd->d];
   if(month < 0 || day < 0) return (ERROR);
   if(month > 12) return (ERROR);
   if(day > 31) return (ERROR);
-  *key = (double) ((366 * (double) num[ymd->y]) + (31 * month) + day);
+  *key = (DOUBLE) ((366 * (DOUBLE) num[ymd->y]) + (31 * month) + day);
   return(SUCCESS);
 }
 
-
 /* Given a time string, write the equivalent double into key. */
 
 int
-GetTimeKey(wchar_t *field, double *key)
+GetTimeKey(wchar_t *field, DOUBLE *key)
 {
   int hours;
   int minutes;
-  int seconds;
+  int iseconds;
+  double seconds;
   UTF8 *cfield;
+  int FieldsFound;
 
   extern UTF8 * ws2u8(wchar_t *);
-  extern unsigned int u8len (UTF8 *);
-
   cfield = ws2u8(field);
   seconds = 0;
-  if(field[u8len(cfield)-3] == '.'){
-    if(sscanf((char *)cfield,"%d:%d.%d",&hours,&minutes,&seconds) != 3) {
-      free(cfield);
-      return(ERROR);
-    }
+  /* HH:MM:SS(.SSS) */
+  FieldsFound = sscanf((char *)cfield,"%d:%d:%lf",&hours,&minutes,&seconds);
+  if(FieldsFound == 3) goto validate;
+
+  /* HH:MM.SS */
+  FieldsFound = sscanf((char *)cfield,"%d:%d.%d",&hours,&minutes,&iseconds);
+  if(FieldsFound == 3) {
+    seconds = (double) iseconds;
+    goto validate;
   }
-  else{
-    if(sscanf((char *)cfield,"%d:%d",&hours,&minutes) != 2) {
-      free(cfield);
-      return(ERROR);
-    }
+
+  /* HH:MM */
+  FieldsFound = sscanf((char *)cfield,"%d:%d",&hours,&minutes);
+  if(FieldsFound == 2) {
+    seconds = 0.0;
+    goto validate;
   }
+
+  free(cfield);  
+  return(ERROR); 
+
+validate:
   free(cfield);
   if(hours < 0 || minutes < 0 || seconds < 0) return (ERROR);
-  if(hours > 24) return(ERROR);
-  if(minutes > 60 || seconds > 60) return (ERROR);
-	
-  *key = (double) (((((double) hours * 60) + (double) minutes) * 60) + (double) seconds);
+  if(hours > 24 || minutes > 59 || seconds >= 60) return (ERROR);
+  *key = (DOUBLE) (((((DOUBLE) hours * 60) + (DOUBLE) minutes) * 60) + (DOUBLE) seconds);
+  return(SUCCESS);
+}
+
+/* Given an angle string, write the equivalent double into key. */
+
+int
+GetAngleKey(wchar_t *field, DOUBLE *key)
+{
+  int degrees;
+  int minutes;
+  double seconds;
+  UTF8 *cfield;
+  int FieldsFound;
+  short NegativeP = 0;
+  DOUBLE Value;
+
+  extern UTF8 * ws2u8(wchar_t *);
+
+  cfield = ws2u8(field);
+  FieldsFound = sscanf((char *)cfield,"%4d:%2d:%lf",&degrees,&minutes,&seconds);
+  if(FieldsFound!=3) {
+    FieldsFound = sscanf((char *)cfield,"%4d %2d %lf",&degrees,&minutes,&seconds);
+  }
+  free(cfield);
+  if(FieldsFound != 3)return(ERROR);
+  if(minutes < 0 || seconds < 0) return (ERROR);
+  if( (minutes > 59) || (seconds >= 60)) return (ERROR);
+
+  if(degrees < 0) {
+    degrees *= (-1);
+    NegativeP = 1;
+  }
+  degrees %= 360;
+  if(NegativeP) degrees = 360 -degrees;
+  Value = ((((DOUBLE) degrees * 60) + (DOUBLE) minutes) * 60) + seconds;
+  *key = Value;
   return(SUCCESS);
 }
 
@@ -242,7 +295,7 @@ IdentifySelf(FILE *fp)
   extern char* compdate;
 	
   fprintf(fp,"%s %s\n",progname,version);
-  fprintf(fp,gettext("Compiled %s\n"),compdate);
+  fprintf(fp,_("Compiled %s\n"),compdate);
 }
 
 FILE *
@@ -255,13 +308,13 @@ OpenFile(char *file,char *mode,char *pgname)
   else{
     switch(*mode){
     case 'r': 
-      fprintf(stderr,gettext("%s: can't open file %s for reading.\n"),pgname,file);
+      fprintf(stderr,_("%s: can't open file %s for reading.\n"),pgname,file);
       break;
     case 'w':
-      fprintf(stderr,gettext("%s: can't open file %s for writing.\n"),pgname,file);
+      fprintf(stderr,_("%s: can't open file %s for writing.\n"),pgname,file);
       break;
     default:
-      fprintf(stderr,gettext("%s: can't open file %s for appending.\n"),pgname,file);
+      fprintf(stderr,_("%s: can't open file %s for appending.\n"),pgname,file);
     }
     return(NULL);
   }
@@ -284,9 +337,9 @@ EvalEscapes(wchar_t *s)
 {
   int state;
   wchar_t c;
-  wchar_t odigit1;
-  wchar_t odigit2;
-  wchar_t sum;
+  wchar_t odigit1 = 0L;
+  wchar_t odigit2 = 0L;
+  wchar_t sum = 0L;
   wchar_t *SStart;
   wchar_t *TStart;
   wchar_t *t;
@@ -453,62 +506,6 @@ EvalEscapes(wchar_t *s)
   return(SUCCESS);
 }
 
-/*
- * Given a number represented as a string, insert delimiters to break it up for
- * readability. Normally, the delimiter will be a comma which will be inserted every
- * three digits. However, the delimiter and groupsize are arguments, permitting use
- * in other locales.
- * 
- * The string is assumed to consist of digits, possibly preceded by spaces,
- * and possibly containing a decimal point, i.e.: [:space:]*[:digit:]*\.[:digit:]*
- * 
- * The delimited string is returned in freshly allocated memory, or NULL storage
- * cannot be allocated. 
- */
-
-char *DelimitNumber(char *number, char delim, int groupsize)
-{
-  int newsize;			/* Characters needed for delimited string */
-  int length;			/* Length of input string */
-  int commas;			/* Number of delimiters to insert */
-  int commacnt;			/* Number of delimiters inserted so far */
-  int digits;			/* Number of digits preceding decimal point */
-  int cnt;			/* Number of digits processed */
-  char *new;			/* String in which to write delimited number */
-  char *point;			/* Location of decimal point */
-  char *pos;			/* Current position in original number */
-  char *npos;			/* Current position in new number */
-  char *NonSpaceNumber;		/* Location of first non-space character in number*/
-
-  length = strlen(number);
-  point = strrchr(number,'.');
-  if(point == NULL) point = number+(length-1);
-  else point = point -1;	/* Point now points at last digit preceding decimal */
-  NonSpaceNumber = strrchr(number,040);
-  if(NonSpaceNumber == NULL) NonSpaceNumber = number;
-  else NonSpaceNumber+=1;
-  digits = 1+point - NonSpaceNumber;
-  commas = (digits-1)/groupsize;
-
-  newsize = length + commas +1;	/* Add one for final null */
-  new = (char *) malloc((size_t)newsize * sizeof(char));
-  if(new){
-    /* First copy right hand part of number, up to and including decimal point */
-    pos = number+length+1;
-    npos = new+newsize;
-    for(; pos > point;) *npos-- = *pos--;
-
-    /* Now copy lefthand part, inserting delimiters as we go */
-    for(cnt=commacnt=0; pos >= NonSpaceNumber;){
-      *npos-- = *pos--;
-      cnt++;
-      if( (cnt % groupsize == 0) && (commacnt++ < commas )) *npos-- = delim;
-    }
-    while(pos >= number) *npos-- = *pos--; /* Copy leading spaces, if any */
-  }
-  return new;
-}
-
 /*
  * Overwrite a textual key with itself, skipping characters specified
  * in the exclusion table. This version differs from the main Exclude
@@ -526,7 +523,7 @@ ExcludeChar(wchar_t *str, wchar_t **out, int len, wchar_t *exct)
   if(OutStr != NULL) free((void *) OutStr);
   OutStr = (wchar_t *) malloc ( (size_t) (len+1) * sizeof(wchar_t));
   if (OutStr == NULL) {
-    fprintf(stderr,gettext("ExcludeChar: out of memory\n"));
+    fprintf(stderr,_("ExcludeChar: out of memory\n"));
     exit(OUTOFMEMORY);
   }
   if(!(exct[str[0]] & EX_INITIAL) ){
@@ -543,3 +540,4 @@ ExcludeChar(wchar_t *str, wchar_t **out, int len, wchar_t *exct)
   *out = OutStr;
   return(cnt);
 }
+
